-- Import necessary modules and libraries
require "import"
import "android.app.*"
import "android.content.pm.*"
import "android.widget.*"
import "android.view.*"
import "android.content.*"
import "java.io.*"
import "android.net.*"

import "android.os.Environment"

local activity = activity
activity.title = "applications manager"

local layout = LinearLayout(activity)
layout.setOrientation(LinearLayout.VERTICAL)
layout.setBackgroundColor(0xFFEDEDED) -- Light gray background
layout.setPadding(16, 16, 16, 16)    -- Add some padding

-- Create an AutoCompleteTextView for search input
local searchEditText = AutoCompleteTextView(activity)
searchEditText.setHint("Enter search keyword")
searchEditText.setPadding(8, 8, 8, 8)  -- Add padding to the search box
searchEditText.setTextSize(16)          -- Increase text size
searchEditText.setTextColor(0xFF000000) -- Dark text color
layout.addView(searchEditText)

-- Create a ListView to display all applications
local listView = ListView(activity)
listView.setPadding(0, 16, 0, 0)   -- Add top margin to the ListView
listView.setBackgroundColor(0xFFFFFFFF) -- White background
layout.addView(listView)

-- Create a PackageManager
local packageManager = activity.getPackageManager()

-- Get a list of installed applications
local applications = packageManager.getInstalledApplications(PackageManager.GET_META_DATA)

-- Create a table to store application names and their package names
local appData = {}

-- Extract application names and add them to the table (openable apps)
for i = 1, #applications do
    local appInfo = applications[i - 1]
    local appName = appInfo.loadLabel(packageManager)
    local appPackageName = appInfo.packageName

    -- Check if the app has a launch intent (openable)
    local launchIntent = packageManager.getLaunchIntentForPackage(appPackageName)
    if launchIntent then
        appData[appName] = appPackageName
    end
end

-- Sort the appData table by application names
local sortedAppData = {}
for k, v in pairs(appData) do
    table.insert(sortedAppData, k)
end
table.sort(sortedAppData)

-- Create an adapter for the ListView to initially show all apps
local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, sortedAppData)
listView.setAdapter(adapter)

-- Create a new adapter for the AutoCompleteTextView
local autoCompleteAdapter = ArrayAdapter(activity, android.R.layout.simple_dropdown_item_1line, sortedAppData)
searchEditText.setAdapter(autoCompleteAdapter)

-- Set a text change listener for the AutoCompleteTextView
searchEditText.addTextChangedListener{
    onTextChanged = function(s, start, before, count)
        local searchText = s.toString()
        local filteredList = {}

        -- Filter the app list based on the search text
        for _, appName in ipairs(sortedAppData) do
            if string.lower(appName):find(string.lower(searchText)) then
                table.insert(filteredList, appName)
            end
        end

        if #filteredList > 0 then
            -- If there are matching apps, display them in the ListView
            local filteredAdapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, filteredList)
            listView.setAdapter(filteredAdapter)
        else
            -- If no matching apps, reset to show all apps
            listView.setAdapter(adapter)
        end
    end
}

-- Set the layout as the content view
--activity.setContentView(layout)

-- Set a click listener for the ListView items to open the selected app
listView.onItemClick = function(parent, view, position, id)
    local selectedAppName = sortedAppData[position + 1]
    local selectedAppPackageName = appData[selectedAppName]
    
    -- Get the intent to launch the selected app
    local launchIntent = packageManager.getLaunchIntentForPackage(selectedAppPackageName)
    
    if launchIntent then
        activity.startActivity(launchIntent)
    else
        -- Handle the case where the app cannot be launched
        toast("Unable to launch the selected app.")
    end
end

local function displayComponents(selectedAppPackageName)
    local packageManager = activity.getPackageManager()
    local packageInfo = packageManager.getPackageInfo(selectedAppPackageName, PackageManager.GET_ACTIVITIES)

    local componentNames = {} -- Create an array to store component names

    if packageInfo.activities then
        for i = 1, #packageInfo.activities do
            local activityInfo = packageInfo.activities[i - 1]
            componentNames[i] = "Activity: " .. activityInfo.name
        end
    end

    if packageInfo.services then
        for i = 1, #packageInfo.services do
            local serviceInfo = packageInfo.services[i - 1]
            componentNames[#componentNames + 1] = "Service: " .. serviceInfo.name
        end
    end

    if packageInfo.receivers then
        for i = 1, #packageInfo.receivers do
            local receiverInfo = packageInfo.receivers[i - 1]
            componentNames[#componentNames + 1] = "Receiver: " .. receiverInfo.name
        end
    end

    if packageInfo.providers then
        for i = 1, #packageInfo.providers do
            local providerInfo = packageInfo.providers[i - 1]
            componentNames[#componentNames + 1] = "Provider: " .. providerInfo.name
        end
    end

    local items = {} -- Create an array of strings to store component names
    for i = 1, #componentNames do
        items[i] = componentNames[i]
    end

    -- Create an AlertDialog with the component names
    local builder = AlertDialog.Builder(activity)
    builder.setTitle("Components for " .. selectedAppPackageName)
    builder.setItems(items, DialogInterface.OnClickListener {
        onClick = function(dialog, which)
            -- When a component is clicked, get the component name
            local selectedComponent = componentNames[which + 1]

            -- Extract the component type and name
            local componentType, componentName = selectedComponent:match("([^:]+):%s(.+)")
            
            if componentType and componentName then
                -- Create an intent to open the selected component
                local intent = Intent()
                intent.setComponent(ComponentName(selectedAppPackageName, componentName))
                intent.setAction(Intent.ACTION_MAIN)
                intent.addCategory(Intent.CATEGORY_LAUNCHER)
                
                -- Try to start the selected component
                if intent.resolveActivity(packageManager) then
                    activity.startActivity(intent)
                else
                    -- Handle the case where the component cannot be launched
                    toast("Unable to launch the selected component.")
                end
            end
        end
    })

    builder.setPositiveButton("OK", nil)

    local dialog = builder.create()
    dialog.show()
end

local function showAppInfo(selectedAppPackageName)
    -- Display application info for the selected app
    local intent = Intent()
    intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
    intent.setData(Uri.fromParts("package", selectedAppPackageName, nil))
    activity.startActivity(intent)
end

local function uninstallApp(selectedAppPackageName)
    -- Uninstall the selected app with a confirmation dialog
    local uninstallIntent = Intent(Intent.ACTION_UNINSTALL_PACKAGE)
    uninstallIntent.setData(Uri.fromParts("package", selectedAppPackageName, nil))
    activity.startActivity(uninstallIntent)
end

local function sharePlayStoreLink(selectedAppPackageName)
    -- Construct the Play Store link for the selected app
    local playStoreLink = "https://play.google.com/store/apps/details?id=" .. selectedAppPackageName
-- Create an intent to send a text message
    local sendIntent = Intent()
    sendIntent.setAction(Intent.ACTION_SEND)
    sendIntent.putExtra(Intent.EXTRA_TEXT, playStoreLink)
    sendIntent.setType("text/plain")
-- Start the activity to share the Play Store link
    activity.startActivity(Intent.createChooser(sendIntent, "Share via"))
end

local function openInPlayStore(selectedAppPackageName)
    -- Open the selected app in the Play Store
    local intent = Intent(Intent.ACTION_VIEW)
    intent.setData(Uri.parse("market://details?id=" .. selectedAppPackageName))
    intent.setPackage("com.android.vending")  -- This line sets the package name for the Play Store app
    activity.startActivity(intent)
end

local function getPackageName(selectedAppPackageName)
    -- Create an AlertDialog to display the package name
    local builder = AlertDialog.Builder(activity)
    builder.setTitle("Package Name")
    builder.setMessage("Package Name: " .. selectedAppPackageName)
    builder.setPositiveButton("OK", nil)  -- Add a simple "OK" button
local dialog = builder.create()
    dialog.show()
end

-- Set a long-click listener for the ListView items to display a popup menu
listView.onItemLongClick = function(parent, view, position, id)
    local selectedAppName = sortedAppData[position + 1]
    local selectedAppPackageName = appData[selectedAppName]

    local popupMenu = PopupMenu(activity, view)
    popupMenu.menu.add("Display Activities")
    popupMenu.menu.add("Application Info")
    popupMenu.menu.add("Uninstall")
    popupMenu.menu.add("Share")
    popupMenu.menu.add("Open in Play Store")
    popupMenu.menu.add("Get Package Name")
    
    popupMenu.setOnMenuItemClickListener(PopupMenu.OnMenuItemClickListener {
        onMenuItemClick = function(item)
            if item.getTitle() == "Display Activities" then
                displayComponents(selectedAppPackageName)
            elseif item.getTitle() == "Application Info" then
                showAppInfo(selectedAppPackageName)
            elseif item.getTitle() == "Uninstall" then
                uninstallApp(selectedAppPackageName)
            elseif item.getTitle() == "Share" then
                sharePlayStoreLink(selectedAppPackageName)
            elseif item.getTitle() == "Open in Play Store" then
                openInPlayStore(selectedAppPackageName)
            elseif item.getTitle() == "Get Package Name" then
                getPackageName(selectedAppPackageName)
            end
            return true
        end
    })
    
    popupMenu.show()
    return true
end

-- Set the layout as the content view
activity.setContentView(layout)

-- Set a click listener for the ListView items to open the selected app
listView.onItemClick = function(parent, view, position, id)
    local selectedAppName = sortedAppData[position + 1]
    local selectedAppPackageName = appData[selectedAppName]
    
    -- Get the intent to launch the selected app
    local launchIntent = packageManager.getLaunchIntentForPackage(selectedAppPackageName)
    
    if launchIntent then
        activity.startActivity(launchIntent)
    else
        -- Handle the case where the app cannot be launched
        toast("Unable to launch the selected app.")
    end
end

local function displayComponents(selectedAppPackageName)
    local packageManager = activity.getPackageManager()
    local packageInfo = packageManager.getPackageInfo(selectedAppPackageName, PackageManager.GET_ACTIVITIES)

    local componentNames = {} -- Create an array to store component names

    if packageInfo.activities then
        for i = 1, #packageInfo.activities do
            local activityInfo = packageInfo.activities[i - 1]
            componentNames[i] = "Activity: " .. activityInfo.name
        end
    end

    if packageInfo.services then
        for i = 1, #packageInfo.services do
            local serviceInfo = packageInfo.services[i - 1]
            componentNames[#componentNames + 1] = "Service: " .. serviceInfo.name
        end
    end

    if packageInfo.receivers then
        for i = 1, #packageInfo.receivers do
            local receiverInfo = packageInfo.receivers[i - 1]
            componentNames[#componentNames + 1] = "Receiver: " .. receiverInfo.name
        end
    end

    if packageInfo.providers then
        for i = 1, #packageInfo.providers do
            local providerInfo = packageInfo.providers[i - 1]
            componentNames[#componentNames + 1] = "Provider: " .. providerInfo.name
        end
    end

    local items = {} -- Create an array of strings to store component names
    for i = 1, #componentNames do
        items[i] = componentNames[i]
    end

    -- Create an AlertDialog with the component names
    local builder = AlertDialog.Builder(activity)
    builder.setTitle("Components for " .. selectedAppPackageName)
    builder.setItems(items, DialogInterface.OnClickListener {
        onClick = function(dialog, which)
            -- When a component is clicked, get the component name
            local selectedComponent = componentNames[which + 1]

            -- Extract the component type and name
            local componentType, componentName = selectedComponent:match("([^:]+):%s(.+)")
            
            if componentType and componentName then
                -- Create an intent to open the selected component
                local intent = Intent()
                intent.setComponent(ComponentName(selectedAppPackageName, componentName))
                intent.setAction(Intent.ACTION_MAIN)
                intent.addCategory(Intent.CATEGORY_LAUNCHER)
                
                -- Try to start the selected component
                if intent.resolveActivity(packageManager) then
                    activity.startActivity(intent)
                else
                    -- Handle the case where the component cannot be launched
                    toast("Unable to launch the selected component.")
                end
            end
        end
    })

    builder.setPositiveButton("OK", nil)

    local dialog = builder.create()
    dialog.show()
end

local function showAppInfo(selectedAppPackageName)
    -- Display application info for the selected app
    local intent = Intent()
    intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
    intent.setData(Uri.fromParts("package", selectedAppPackageName, nil))
    activity.startActivity(intent)
end

local function uninstallApp(selectedAppPackageName)
    -- Uninstall the selected app with a confirmation dialog
    local uninstallIntent = Intent(Intent.ACTION_UNINSTALL_PACKAGE)
    uninstallIntent.setData(Uri.fromParts("package", selectedAppPackageName, nil))
    activity.startActivity(uninstallIntent)
end

local function sharePlayStoreLink(selectedAppPackageName)
    -- Construct the Play Store link for the selected app
    local playStoreLink = "https://play.google.com/store/apps/details?id=" .. selectedAppPackageName
-- Create an intent to send a text message
    local sendIntent = Intent()
    sendIntent.setAction(Intent.ACTION_SEND)
    sendIntent.putExtra(Intent.EXTRA_TEXT, playStoreLink)
    sendIntent.setType("text/plain")
-- Start the activity to share the Play Store link
    activity.startActivity(Intent.createChooser(sendIntent, "Share via"))
end

local function openInPlayStore(selectedAppPackageName)
    -- Open the selected app in the Play Store
    local intent = Intent(Intent.ACTION_VIEW)
    intent.setData(Uri.parse("market://details?id=" .. selectedAppPackageName))
    intent.setPackage("com.android.vending")  -- This line sets the package name for the Play Store app
    activity.startActivity(intent)
end

local function getPackageName(selectedAppPackageName)
    -- Create an AlertDialog to display the package name
    local builder = AlertDialog.Builder(activity)
    builder.setTitle("Package Name")
    builder.setMessage("Package Name: " .. selectedAppPackageName)
    builder.setPositiveButton("OK", nil)  -- Add a simple "OK" button
local dialog = builder.create()
    dialog.show()
end

local function shareApk(selectedAppPackageName)
    local packageManager = activity.getPackageManager()

    -- Get the package info for the selected app
    local packageInfo = packageManager.getPackageInfo(selectedAppPackageName, PackageManager.GET_META_DATA)

    if packageInfo.applicationInfo.sourceDir then
        -- Get the path to the original APK file
        local apkPath = packageInfo.applicationInfo.sourceDir

        -- Get the application name
        local appName = packageManager.getApplicationLabel(packageInfo.applicationInfo)

        -- Remove any special characters from the app name and extract the version name
        local cleanAppName = appName:gsub("[^a-zA-Z0-9]+", "_")
        local versionName = packageInfo.versionName

        -- Create the file name with app name and version
        local fileName = cleanAppName .. "_v" .. versionName .. ".apk"

        -- Create a directory for the APK in external storage
        local externalDir = File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS), "shared_apk")
        externalDir.mkdirs()

        -- Copy the APK file to the external directory with the selected app's name and version
        local sharedApkFile = File(externalDir, fileName)
        LuaUtil.copyFile(apkPath, sharedApkFile.getAbsolutePath())

        -- Create an intent to send the APK file
        local sendIntent = Intent()
        sendIntent.setAction(Intent.ACTION_SEND)
        sendIntent.setType("application/vnd.android.package-archive")
        sendIntent.putExtra(Intent.EXTRA_STREAM, activity.getUriForFile(sharedApkFile))
        sendIntent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION) -- Grant read permissions

        activity.startActivity(Intent.createChooser(sendIntent, "Share APK File"))
    end
end

-- Function to get API information for the selected app
local function getApiInfo(selectedAppPackageName)
    local packageManager = activity.getPackageManager()

    -- Get the package info for the selected app
    local packageInfo = packageManager.getPackageInfo(selectedAppPackageName, PackageManager.GET_META_DATA)

    if packageInfo.applicationInfo.sourceDir then
        -- Get the path to the APK file
        local apkPath = packageInfo.applicationInfo.sourceDir

        -- Create a directory for extracting the APK
        local extractDir = File(activity.getExternalFilesDir(nil), "extracted_apk")
        extractDir.mkdirs()

        -- Unzip the APK to the extractDir
        LuaUtil.unZip(apkPath, extractDir.getAbsolutePath())

        -- Now you can analyze the extracted files in the extractDir
        -- You can browse and analyze the classes, methods, and fields

local function shareUnzippedApk(extractDir)
    local apkFiles = extractDir.listFiles() -- Get a list of files in the extracted directory
    local shareIntent = Intent(Intent.ACTION_SEND_MULTIPLE)
    shareIntent.setType("application/vnd.android.package-archive")

    local uris = ArrayList() -- Create an ArrayList to hold the file URIs

    -- Iterate over the Java array of files using a Java loop
    for i = 0, #apkFiles - 1 do
        local file = apkFiles[i]
        local uri = activity.getUriForFile(file)
        uris.add(uri)
    end

    shareIntent.putParcelableArrayListExtra(Intent.EXTRA_STREAM, uris)
    activity.startActivity(Intent.createChooser(shareIntent, "Share APK Files"))
end

        local builder = AlertDialog.Builder(activity)
        builder.setTitle("API Info for " .. selectedAppPackageName)
        builder.setMessage("APK Extracted to: " .. extractDir.getAbsolutePath())
        builder.setPositiveButton("Share APK", DialogInterface.OnClickListener {
            onClick = function(dialog, which)
                shareUnzippedApk(extractDir) -- Call the function to share the unzipped APK
            end
        })
        builder.setNegativeButton("OK", nil) -- Add an "OK" button

        local dialog = builder.create()
        dialog.show()
    else
        -- If no APK path is available, display a message
        local builder = AlertDialog.Builder(activity)
        builder.setTitle("API Info for " .. selectedAppPackageName)
        builder.setMessage("No APK path available.")
        builder.setPositiveButton("OK", nil) -- Add an "OK" button

        local dialog = builder.create()
        dialog.show()
    end
end

-- Set a long-click listener for the ListView items to display a popup menu
listView.onItemLongClick = function(parent, view, position, id)
    local selectedAppName = sortedAppData[position + 1]
    local selectedAppPackageName = appData[selectedAppName]
local popupMenu = PopupMenu(activity, view)
    popupMenu.menu.add("Display Activities")
    popupMenu.menu.add("Application Info")
    popupMenu.menu.add("Uninstall")
    popupMenu.menu.add("Share")
    popupMenu.menu.add("Open in Play Store")
    popupMenu.menu.add("Get Package Name")
    popupMenu.menu.add("Share APK") -- Add the "Share APK" option
   -- popupMenu.menu.add("Get API Info") -- Add the "Get API Info" option
popupMenu.setOnMenuItemClickListener(PopupMenu.OnMenuItemClickListener {
        onMenuItemClick = function(item)
            if item.getTitle() == "Display Activities" then
                displayComponents(selectedAppPackageName)
            elseif item.getTitle() == "Application Info" then
                showAppInfo(selectedAppPackageName)
            elseif item.getTitle() == "Uninstall" then
                uninstallApp(selectedAppPackageName)
            elseif item.getTitle() == "Share" then
                sharePlayStoreLink(selectedAppPackageName)
            elseif item.getTitle() == "Open in Play Store" then
                openInPlayStore(selectedAppPackageName)
            elseif item.getTitle() == "Get Package Name" then
                getPackageName(selectedAppPackageName)
            elseif item.getTitle() == "Share APK" then -- Handle the "Share APK" option
                shareApk(selectedAppPackageName)
            elseif item.getTitle() == "Get API Info" then
                getApiInfo(selectedAppPackageName) -- Handle the "Get API Info" option
            end
            return true
        end
    })
popupMenu.show()
    return true
end

-- Set the layout as the content view
activity.setContentView(layout)